#include "BarinkEngine.h"
#include "Scene\SceneManager.h"
#include "Scene\SceneNodeTypes.h"
#include "AssetManager/ModelImporter.h"

#include "imgui.h"
#include "GUI.h"
#include "Util.h"

/*
* Define globals
*/
Shader* shader;

char* code = new char[254];

const std::string vertexShaderSource = "../build/SandboxApplication/Debug/test.vs";
const std::string fragmentShaderSource = "../build/SandboxApplication/Debug/test.fs";

BarinkEngine::ModelImporter* MI = new BarinkEngine::ModelImporter();

Scene* Level1;
BarinkEngine::SceneObject* cube;
/*
* Runs once at startup 
* - USe to initialize the game/sandbox/demo
*/
void Start() {
    // Build a basic test scene 
    // NOTE: This will later be done through an editor 

    // Create a level and load it as the current level
    std::string levelName("Test Level");
    Level1 = SceneManager::CreateScene(levelName);
    SceneManager::LoadScene(*Level1);
    
    shader = new Shader(vertexShaderSource, fragmentShaderSource);

    // Create a cube node
   
    cube = MI->Import("../build/SandboxApplication/Debug/Models/cube.obj");
    cube->renderable->material = new Material(*shader);
    cube->renderable->material->Color = glm::vec3(1.0f, 0.0f, 0.0f);

    // What is in cube now ??    
    std::cout <<  "mesh vertices: " << cube->renderable->mesh->vertices.size() << std::endl;
    std::cout << "mesh elements: " << cube->renderable->mesh->elements.size() << std::endl;

    Level1->GetRoot().addChild(*cube);

    memset(code, '\0', 254);

    // TODO: Move to runtime/ Engine 
    // NOTE: Submits should later be done through walking the sceneTree
    renderer.Submit(cube->renderable);


}

/*
* Runs every frame
* - Use to draw Immediate mode graphics (Not meant for HUD's )
*/
void ImmediateGraphicsDraw() {
    ImGui::NewFrame();

    // Show ImGui demo such that I can easily look
    // at possible GUI elements to use
   // ImGui::ShowDemoWindow();

    // Show internal BarinkEngine stats
    ShowStats();

    // Show different tooling for this specific sandbox
   // CameraTool(cam);
    //ScriptingTool(code);

    //SceneExplorer(*Level1, "Scene Explorer");

}

/*
* Runs every frame
* - Meant for game logic ( non-physics related) 
*/
void Update()
{

}

/*
* Runs at the end of the program
* - Meant for cleanup
*/
void Stop() {
    delete MI;
    delete shader;
}